import json
from dataclasses import dataclass
from typing import List, Dict, Iterable

import requests
from bs4 import BeautifulSoup

# =================================================
from utils import cloaking
from model.paper import Paper
from parsers.ppaper import PaperParser
from persistence.encoder import EnhancedJSONEncoder


@dataclass
class Profile(object):
    """
    Profile is a researcher's profile:
    . name: The name of the researcher
    . citations: The number of citations for that researcher
    . labels: Author's associated keywords
    . query: The Google Scholar query to the researchers Google Scholar Profile
    . user: The Google Scholar researcher unique identifier
    . papers: The list of papers associated with the profile
    """

    name: str
    citations: int
    labels: List[str]
    query: str
    user: str
    university: str
    papers: Iterable[Paper] = None

    def search_papers(self, return_papers=False):
        """
        Sends GET request for given Profile, using the user attribute.
        """
        url = "https://scholar.google.ca/citations"
        soup = BeautifulSoup(u'', 'html.parser')
        cstart: int = 0
        soups: List[BeautifulSoup] = []
        while not soup.find("td", {"class": "gsc_a_e"}):
            payload: Dict[str, str] = {
                "hl": "en",
                "user": self.user,
                "cstart": cstart,
                "pagesize": "100"
            }
            headers = cloaking.headers_rand_user_agent()
            r = requests.get(url,
                             params=payload,
                             headers=headers,
                             timeout=5)
            r.close()
            cloaking.delay()
            soup = BeautifulSoup(r.content, "html.parser")
            soups.extend(soup.find_all("tr", {"class": "gsc_a_tr"}))
            cstart += 100
        self.papers = PaperParser.parse(soups)
        if return_papers:
            return self.papers

    def serialize(self) -> str:
        return json.dumps(self, cls=EnhancedJSONEncoder)

    def to_json(self, filename: str):
        if not filename.endswith(".json"):
            filename += ".json"
        if self.papers:
            self.papers = list(self.papers)
        with open(filename, "w") as outfile:
            json.dump(self, outfile, cls=EnhancedJSONEncoder)
