from typing import (
    List,
    Dict,
    Iterable,
    Callable,
    Any
)

from bs4 import BeautifulSoup

# =================================================
from model.paper import Paper


class PaperParser(object):
    """
    PaperParser encapsulates the parsing behaviour required to instantiate a Paper object

    staticmethods:
        parse       -- Converts a list of BeautifulSoup instances into an iterable collection of Paper
        make_parser -- Converts a BeautifulSoup instance into a Paper instance
        extract     -- Converts a BeautifulSoup instance into an integer or None.
    """

    @staticmethod
    def parse(soups: List[BeautifulSoup]) -> Iterable[Paper]:
        """
        Convert a list of BeautifulSoup instances into a list of Paper instances.

        keyword arguments:
            soups -- list of BeautifulSoup instances.
        """
        # If using decorator, remove [:-1]
        for row in soups[:-1]:
            yield PaperParser.make_paper(row)

    @staticmethod
    def make_paper(table_soup: BeautifulSoup) -> Paper:
        """
        Convert a BeautifulSoup HTML row instance into a Paper instance.

        keyword arguments:
            table_soup -- BeautifulSoup instance of an HTML row element.
        """
        title: str = table_soup.find("td", {"class": "gsc_a_t"}).a.text
        citations: int or None = PaperParser.extract(table_soup, "td", {"class": "gsc_a_c"}, int)
        pub_year: int or None = PaperParser.extract(table_soup, "td", {"class": "gsc_a_y"}, int)
        return Paper(title=title, citations=citations, pub_year=pub_year)

    @staticmethod
    def extract(soup: BeautifulSoup, tag: str, attr: Dict[str, str], func: Callable[[Any], Any]) -> int or None:
        """
        Converts a BeautifulSoup instance into an integer or None.

        keyword arguments;
            soup -- A BeautifulSoup instance
            tag  -- A HTML tag
            attr -- HTML tag attributes
            func -- A function object to be called
        """
        text = soup.find(tag, attr).text
        try:
            return func(text)
        except ValueError:
            return None
