import re
from typing import List

from bs4 import BeautifulSoup

# =================================================
from model.profile import Profile


class ProfileParser(object):
    """
    ProfileParser encapsulates the parsing behaviour requires to instantiate a Profile object

    staticmethods:
        parse -- Converts BeautifulSoup instance into a list of Profile
    """

    user_pattern = re.compile("user=(.*)")
    cited_pattern = re.compile("Cited by (\d{,})")

    @staticmethod
    def parse(soup: BeautifulSoup) -> List[Profile]:
        """
        Converts a BeautifulSoup instance into list of Profile.

        keyword arguments:
            soup -- BeautifulSoup instance
        """
        names: List[str] = [name.get_text()
                            for name in soup.find_all("h3", {"class": "gs_ai_name"})]
        citations: List[int] = [int(re.search(ProfileParser.cited_pattern, hit.text).group(1))
                                if re.match(ProfileParser.cited_pattern, hit.text) else 0
                                for hit in soup.find_all("div", {"class": "gs_ai_cby"})]
        labels: List[List[str]] = [
            [label.text for label in soup.find_all("a", {"class": "gs_ai_one_int"})]
            for soup in soup.find_all("div", {"class": "gs_ai gs_scl gs_ai_chpr"})
        ]
        queries: List[str] = [author.a["href"].replace("/citations", "")
                              for author in soup.find_all("div", {"class": "gs_ai gs_scl gs_ai_chpr"})]
        users: List[str] = [re.search(ProfileParser.user_pattern, query).group(1)
                            for query in queries]
        universities: List[str] = [university.text
                                   if university is not None else None
                                   for university in soup.find_all("div", {"class": "gs_ai_aff"})]
        # Add labels after citations
        details = zip(names, citations, labels, queries, users, universities)
        return [Profile(*detail) for detail in details]
