import re
from typing import (
    List,
    Dict,
    Iterable,
    Callable
)

from random import choice
import requests
from bs4 import BeautifulSoup

from model.profile import Profile
from parsers.pprofile import ProfileParser
# =================================================
from utils import cloaking


class GoogleScholar(object):
    """
    GoogleScholar is an encapsulation for user interaction with the Google Scholar search engine via python

    methods:
        search_profile -- Allows for searching Google Scholar Profiles
        search_user_id -- Allows for searching Google Scholar Profiles using unique identifiers
    
    staticmethods:
        filter_by_citations: Identifies most or least cited Profile in a list of Profile
    """

    url: str = "https://scholar.google.ca/citations"
    query_pattern = re.compile("citations(\?.*)")

    def __init__(self, proxies = None):
        self.proxies = list(proxies)

    def search_profile(self, *args, cited: str = None, simplify: bool = False) -> List[Profile] or Profile:
        """
        Sends GET request for given query strings.

        keyword arguments:
            args     -- Query string of interest (ex. author name, ccTLD, etc)
            cited    -- Output modifier, return most or least cited Profile, or all Profiles. (default is None)
            simplify -- If the query results in 1 Profile, do not return it as a list

        Instruction:
            To search by using labels, prefix your string with "label:" and no space.
                Example -- "label:Ecology"
            `cited` is one of "most", "least" or None.
        TODO:
            Only returns top 10 authors, scrape_papers method does not work
        """
        mauthors = "+".join([arg.replace(" ", "+") for arg in args])
        payload: Dict[str, str] = {
            "view_op": "search_authors",
            "hl": "en",
            "mauthors": mauthors
        }
        r = requests.get(self.url,
                         params=payload,
                         headers=cloaking.HEADER,
                         timeout=5)
        r.close()
        cloaking.delay()
        soup = BeautifulSoup(r.content, "html.parser")
        profiles: List[Profile] = ProfileParser.parse(soup)
        if simplify:
            return profiles[0]
        if cited is None:
            return profiles
        elif cited.lower() == "most":
            return GoogleScholar.filter_by_citations(profiles, max)
        elif cited.lower() == "least":
            return GoogleScholar.filter_by_citations(profiles, min)

    @staticmethod
    def filter_by_citations(profiles: List[Profile], func: Callable[[Iterable[int]], int]) -> Profile:
        """
        Determines index of the most or least cited author.

        keyword arguments:
            profiles -- list of Profile instances.
        """
        citations: List[int] = [profile.citations for profile in profiles]
        return profiles[citations.index(func(citations))]

    def search_user_id(self, user: str) -> Profile:
        """
        Sends GET requests for given user ID.

        keyword arguments:
            user -- A researchers unique identifier.
        """
        payload = {
            "hl": "en",
            "user": user
        }
        headers = cloaking.HEADER
        r = requests.get(self.url,
                         params=payload,
                         headers=headers,
                         proxies={'http': f'http://{choice(self.proxies)}'},
                         timeout=5)
        r.close()
        cloaking.delay()
        soup = BeautifulSoup(r.content, "html.parser")
        name: str = soup.find("div", {"id": "gsc_prf_in"}).get_text()
        try:
            citations: int = soup.find("td", {"class": "gsc_rsb_std"}).get_text()
        except:
            citations = None
        try:
            labels: List[str] = [label.text
                                 for label in soup.find_all("a", {"class": "gsc_prf_inta gs_ibl"}, href=True)]
        except:
            labels = None
        query: str = re.search(GoogleScholar.query_pattern, r.url).group(1)
        try:
            university: str = soup.find("a", {"class": "gsc_prf_ila"}).get_text()
        except:
            university = None
        return Profile(name=name, citations=citations, labels=labels, 
                       query=query, user=user, university=university)


class ResearchGate(object):
    pass


class NSERC(object):
    pass


class Loop(object):
    pass


class WebOfScience(object):
    pass
